@@ -1,10 +1,8 @@
-
-#DES_PP.pm - Pure perl implementation of DES.
-
-
-
-#The master file for the module is DES_PP.ppp which needs to be run through
-#the preprocessor.  Please edit DES_PP.ppp if you need to modify!
+# -*- perl -*-
+# DES_PP.pm - Pure perl implementation of DES.
+#
+# The master file for the module is DES_PP.m4 which needs to be run through
+# the m4.  Please edit DES_PP.m4 if you need to modify!
 
 package Crypt::DES_PP;
 
@@ -19,11 +17,11 @@ require Exporter;
 @ISA = qw (Exporter);
 @EXPORT = qw ();
 @EXPORT_OK = qw ();
-$VERSION = '0.07';
+$VERSION = '1.00';
 
 use constant BLKSIZE => 8;
 
-#Stolen from Crypt::DES.
+# Stolen from Crypt::DES.
 sub usage {
     my ($package, $filename, $line, $subr) = caller (1);
     $Carp::CarpLevel = 2;
@@ -60,36 +58,9 @@ sub decrypt {
 
 use constant ITERATIONS => 16;
 
-#Right-shifting in Perl with use integer is a little tricky.  In the
-#absence of unsigned data types, the sign is always preserved which
-#is undesirable in cryptographic applications.
-use constant RIGHT_SHIFT_MASK => 
-    [
-     0xffffffff, 0x7fffffff, 0x3fffffff, 0x1fffffff,
-     0x0fffffff, 0x07ffffff, 0x03ffffff, 0x01ffffff,
-     0x00ffffff, 0x007fffff, 0x003fffff, 0x001fffff,
-     0x000fffff, 0x0007ffff, 0x0003ffff, 0x0001ffff,
-     0x0000ffff, 0x00007fff, 0x00003fff, 0x00001fff,
-     0x00000fff, 0x000007ff, 0x000003ff, 0x000001ff,
-     0x000000ff, 0x0000007f, 0x0000003f, 0x0000001f,
-     0x0000000f, 0x00000007, 0x00000003, 0x00000001,
-     ];
-
-
-
-
-
-
-
-    
-
-
-    
-#FIXME: Really use that!
-use constant ALT_ECB => 1;
-
-use constant des_SPtrans_x =>
-    [
+# These used to be a single reference to an array of array references.
+# Splitting them up into distinct constants slightly improves performance.
+use constant des_SPtrans_0 =>
     [ # Nibble 0
       0x00820200, 0x00020000, 0x80800000, 0x80820200,
       0x00800000, 0x80020200, 0x80020000, 0x80800000,
@@ -107,7 +78,8 @@ use constant des_SPtrans_x =>
       0x00800000, 0x80000200, 0x80020000, 0x00000000,
       0x00020000, 0x00800000, 0x80800200, 0x00820200,
       0x80000000, 0x80820000, 0x00000200, 0x80020200,
-      ],
+      ];
+use constant des_SPtrans_1 =>
     [ # Nibble 1
       0x10042004, 0x00000000, 0x00042000, 0x10040000,
       0x10000004, 0x00002004, 0x10002000, 0x00042000,
@@ -125,7 +97,8 @@ use constant des_SPtrans_x =>
       0x00000004, 0x10042004, 0x00042000, 0x10040000,
       0x10040004, 0x00040000, 0x00002004, 0x10002000,
       0x10002004, 0x00000004, 0x10040000, 0x00042000,
-      ],
+      ];
+use constant des_SPtrans_2 =>
     [ # Nibble 2
       0x41000000, 0x01010040, 0x00000040, 0x41000040,
       0x40010000, 0x01000000, 0x41000040, 0x00010040,
@@ -143,7 +116,8 @@ use constant des_SPtrans_x =>
       0x00010000, 0x01000040, 0x41000040, 0x00010040,
       0x01000040, 0x00000000, 0x41010000, 0x40000040,
       0x41000000, 0x40010040, 0x00000040, 0x01010000,
-      ],
+      ];
+use constant des_SPtrans_3 =>
     [ # Nibble 3 
       0x00100402, 0x04000400, 0x00000002, 0x04100402,
       0x00000000, 0x04100000, 0x04000402, 0x00100002,
@@ -161,7 +135,8 @@ use constant des_SPtrans_x =>
       0x04100402, 0x00000002, 0x04000400, 0x00100402,
       0x00100002, 0x00100400, 0x04100000, 0x04000402,
       0x00000402, 0x04000000, 0x04000002, 0x04100400,
-      ],
+      ];
+use constant des_SPtrans_4 =>
     [ # Nibble 4
       0x02000000, 0x00004000, 0x00000100, 0x02004108,
       0x02004008, 0x02000100, 0x00004108, 0x02004000,
@@ -179,7 +154,8 @@ use constant des_SPtrans_x =>
       0x02004008, 0x02004100, 0x00000108, 0x00004000,
       0x00004100, 0x02004008, 0x02000100, 0x00000108,
       0x00000008, 0x00004108, 0x02004000, 0x02000008,
-      ],
+      ];
+use constant des_SPtrans_5 =>
     [ # Nibble 5
       0x20000010, 0x00080010, 0x00000000, 0x20080800,
       0x00080010, 0x00000800, 0x20000810, 0x00080000,
@@ -197,7 +173,8 @@ use constant des_SPtrans_x =>
       0x20000010, 0x20080000, 0x00080810, 0x00000000,
       0x00000800, 0x20000010, 0x20000810, 0x20080800,
       0x20080000, 0x00000810, 0x00000010, 0x20080010,
-      ],
+      ];
+use constant des_SPtrans_6 =>
     [ # Nibble 6
       0x00001000, 0x00000080, 0x00400080, 0x00400001,
       0x00401081, 0x00001001, 0x00001080, 0x00000000,
@@ -215,7 +192,8 @@ use constant des_SPtrans_x =>
       0x00000081, 0x00001000, 0x00401081, 0x00400000,
       0x00401080, 0x00000001, 0x00001001, 0x00401081,
       0x00400001, 0x00401080, 0x00401000, 0x00001001,
-      ],
+      ];
+use constant des_SPtrans_7 =>
     [ # Nibble 7
       0x08200020, 0x08208000, 0x00008020, 0x00000000,
       0x08008000, 0x00200020, 0x08200000, 0x08208020,
@@ -233,11 +211,10 @@ use constant des_SPtrans_x =>
       0x00208000, 0x00008020, 0x08008020, 0x08200000,
       0x00000020, 0x08208000, 0x00208020, 0x00000000,
       0x08000000, 0x08200020, 0x00008000, 0x00208020,
-      ],
-    ];
+      ];
 
-use constant des_skb => 
-    [
+# These have also been split up.
+use constant des_skb_0 => 
     [ # For C bits (numbered as per FIPS 46) 1 2 3 4 5 6.
       0x00000000, 0x00000010, 0x20000000, 0x20000010,
       0x00010000, 0x00010010, 0x20010000, 0x20010010,
@@ -255,7 +232,8 @@ use constant des_skb =>
       0x00090020, 0x00090030, 0x20090020, 0x20090030,
       0x00080820, 0x00080830, 0x20080820, 0x20080830,
       0x00090820, 0x00090830, 0x20090820, 0x20090830,
-      ],
+      ];
+use constant des_skb_1 => 
     [ # For C bits (numbered as per FIPS 46) 7 8 10 11 12 13
       0x00000000, 0x02000000, 0x00002000, 0x02002000,
       0x00200000, 0x02200000, 0x00202000, 0x02202000,
@@ -273,7 +251,8 @@ use constant des_skb =>
       0x10200400, 0x12200400, 0x10202400, 0x12202400,
       0x10000404, 0x12000404, 0x10002404, 0x12002404,
       0x10200404, 0x12200404, 0x10202404, 0x12202404,
-      ],
+      ];
+use constant des_skb_2 => 
     [ # For C bits (numbered as per FIPS 46) 14 15 16 17 19 20
       0x00000000, 0x00000001, 0x00040000, 0x00040001,
       0x01000000, 0x01000001, 0x01040000, 0x01040001,
@@ -291,7 +270,8 @@ use constant des_skb =>
       0x09000200, 0x09000201, 0x09040200, 0x09040201,
       0x08000202, 0x08000203, 0x08040202, 0x08040203,
       0x09000202, 0x09000203, 0x09040202, 0x09040203,
-      ],
+      ];
+use constant des_skb_3 => 
     [ # For C bits (numbered as per FIPS 46) 21 23 24 26 27 28
       0x00000000, 0x00100000, 0x00000100, 0x00100100,
       0x00000008, 0x00100008, 0x00000108, 0x00100108,
@@ -309,7 +289,8 @@ use constant des_skb =>
       0x04020008, 0x04120008, 0x04020108, 0x04120108,
       0x04021000, 0x04121000, 0x04021100, 0x04121100,
       0x04021008, 0x04121008, 0x04021108, 0x04121108,
-      ],
+      ];
+use constant des_skb_4 => 
     [ # For D bits (numbered as per FIPS 46) 1 2 3 4 5 6
       0x00000000, 0x10000000, 0x00010000, 0x10010000,
       0x00000004, 0x10000004, 0x00010004, 0x10010004,
@@ -327,7 +308,8 @@ use constant des_skb =>
       0x00101004, 0x10101004, 0x00111004, 0x10111004,
       0x20101000, 0x30101000, 0x20111000, 0x30111000,
       0x20101004, 0x30101004, 0x20111004, 0x30111004,
-      ],
+      ];
+use constant des_skb_5 => 
     [ # For D bits (numbered as per FIPS 46) 8 9 11 12 13 14
       0x00000000, 0x08000000, 0x00000008, 0x08000008,
       0x00000400, 0x08000400, 0x00000408, 0x08000408,
@@ -345,7 +327,8 @@ use constant des_skb =>
       0x02000401, 0x0A000401, 0x02000409, 0x0A000409,
       0x02020001, 0x0A020001, 0x02020009, 0x0A020009,
       0x02020401, 0x0A020401, 0x02020409, 0x0A020409,
-      ],
+      ];
+use constant des_skb_6 => 
     [ # For D bits (numbered as per FIPS 46) 16 17 18 19 20 21
       0x00000000, 0x00000100, 0x00080000, 0x00080100,
       0x01000000, 0x01000100, 0x01080000, 0x01080100,
@@ -363,7 +346,8 @@ use constant des_skb =>
       0x01200200, 0x01200300, 0x01280200, 0x01280300,
       0x00200210, 0x00200310, 0x00280210, 0x00280310,
       0x01200210, 0x01200310, 0x01280210, 0x01280310,
-      ],
+      ];
+use constant des_skb_7 => 
     [ # For D bits (numbered as per FIPS 46) 22 23 24 25 27 28
       0x00000000, 0x04000000, 0x00040000, 0x04040000,
       0x00000002, 0x04000002, 0x00040002, 0x04040002,
@@ -381,21 +365,38 @@ use constant des_skb =>
       0x00000822, 0x04000822, 0x00040822, 0x04040822,
       0x00002820, 0x04002820, 0x00042820, 0x04042820,
       0x00002822, 0x04002822, 0x00042822, 0x04042822,
-      ]
-    ];
+      ];
+
+
+
+
+
+
+
 
 sub expand_key ($) {
     my ($c, $d) = unpack "VV", shift;
+
     usage ("at least 8 byte key") unless defined $d;
     my @k = ();
     
     my ($t, $s);
-    ((   $t ) = (((((( ( $d ) ) >> (  (  4 ) )) &  RIGHT_SHIFT_MASK->[(  (  4 ) )]) ) ^ (  $c )) & (  0x0f0f0f0f )), (  $c ) ^= (   $t ), ( $d ) ^= ((   $t ) << (  4 ))) ;
-    ((  $t ) = (((( $c ) << (16 - (  -2 ))) ^ ( $c )) & (  0xcccc0000 )), ( $c ) = ( $c ) ^ (  $t ) ^ (((( (  $t ) ) >> (  (16 - (  -2 )) )) &  RIGHT_SHIFT_MASK->[(  (16 - (  -2 )) )]) )) ;
-    ((  $t ) = (((( $d ) << (16 - (  -2 ))) ^ ( $d )) & (  0xcccc0000 )), ( $d ) = ( $d ) ^ (  $t ) ^ (((( (  $t ) ) >> (  (16 - (  -2 )) )) &  RIGHT_SHIFT_MASK->[(  (16 - (  -2 )) )]) )) ;
-    ((   $t ) = (((((( ( $d ) ) >> (  (  1 ) )) &  RIGHT_SHIFT_MASK->[(  (  1 ) )]) ) ^ (  $c )) & (  0x55555555 )), (  $c ) ^= (   $t ), ( $d ) ^= ((   $t ) << (  1 ))) ;
-    ((   $t ) = (((((( ( $c ) ) >> (  (  8 ) )) &  RIGHT_SHIFT_MASK->[(  (  8 ) )]) ) ^ (  $d )) & (  0x00ff00ff )), (  $d ) ^= (   $t ), ( $c ) ^= ((   $t ) << (  8 ))) ;
-    ((   $t ) = (((((( ( $d ) ) >> (  (  1 ) )) &  RIGHT_SHIFT_MASK->[(  (  1 ) )]) ) ^ (  $c )) & (  0x55555555 )), (  $c ) ^= (   $t ), ( $d ) ^= ((   $t ) << (  1 ))) ;
+    $t = (($d >> 4) ^ $c) & 0x0f0f0f0f;
+    $c ^= $t;
+    $d ^= $t << 4;
+    $t = (($c << 18) ^ $c) & 0xcccc0000;
+    $c = $c ^ $t ^ (($t >> 18) & 0x00003fff);
+    $t = (($d << 18) ^ $d) & 0xcccc0000;
+    $d = $d ^ $t ^ (($t >> 18) & 0x00003fff);
+    $t = (($d >> 1) ^ $c) & 0x55555555;
+    $c ^= $t;
+    $d ^= $t << 1;
+    $t = (($c >> 8) ^ $d) & 0x00ff00ff;
+    $d ^= $t;
+    $c ^= $t << 8;
+    $t = (($d >> 1) ^ $c) & 0x55555555;
+    $c ^= $t;
+    $d ^= $t << 1;
     $d =     ((($d & 0x000000ff) << 16) |  ($d & 0x0000ff00)     |
 	      (($d >> 16) & 0x000000ff) | (($c >> 4) & 0x0f000000));
     $c &= 0x0fffffff;
@@ -403,9 +404,11 @@ sub expand_key ($) {
     use constant shifts2 => [0, 0, 1, 1, 1, 1, 1, 1, 
 			     0, 1, 1, 1, 1, 1, 1, 0];
     
+    # Do not try to unroll any of the loops (not this one and not the
+    # one in crypt().  It will make things slower (about 30 %!).
     foreach my $i (0 .. ITERATIONS - 1) {
-	#No need to mask out the sign here because only the
-	#lower 28 bits are used.
+	# No need to mask out the sign here because only the
+	# lower 28 bits are used.
 	if (shifts2->[$i]) { 
 	    $c = (($c >> 2) | ($c << 26)); 
 	    $d = (($d >> 2) | ($d << 26));
@@ -416,20 +419,20 @@ sub expand_key ($) {
 	$c &= 0x0fffffff;
 	$d &= 0x0fffffff;
 	
-	$s = (des_skb->[0]->[($c) & 0x3f] |
-	      des_skb->[1]->[(($c >>  6) & 0x03) | 
-			     (($c >>  7) & 0x3c)] |
-	      des_skb->[2]->[(($c >> 13) & 0x0f) | 
-			     (($c >> 14) & 0x30)] |
-	      des_skb->[3]->[(($c >> 20) & 0x01) | 
-			     (($c >> 21) & 0x06) |
-			     (($c >> 22) & 0x38)]); 
-	$t = (des_skb->[4]->[($d) & 0x3f] |
-	      des_skb->[5]->[(($d >>  7) & 0x03) | 
-			     (($d >>  8) & 0x3c)] |
-	      des_skb->[6]->[ ($d >> 15) & 0x3f] |
-	      des_skb->[7]->[(($d >> 21) & 0x0f) | 
-			     (($d >> 22) & 0x30)]);
+	$s = (des_skb_0->[($c) & 0x3f] |
+	      des_skb_1->[(($c >>  6) & 0x03) | 
+			  (($c >>  7) & 0x3c)] |
+	      des_skb_2->[(($c >> 13) & 0x0f) | 
+			  (($c >> 14) & 0x30)] |
+	      des_skb_3->[(($c >> 20) & 0x01) | 
+			  (($c >> 21) & 0x06) |
+			  (($c >> 22) & 0x38)]); 
+	$t = (des_skb_4->[($d) & 0x3f] |
+	      des_skb_5->[(($d >>  7) & 0x03) | 
+			  (($d >>  8) & 0x3c)] |
+	      des_skb_6->[ ($d >> 15) & 0x3f] |
+	      des_skb_7->[(($d >> 21) & 0x0f) | 
+			  (($d >> 22) & 0x30)]);
 	
 	$k[$i << 1] = (($t << 16) | ($s & 0x0000ffff)) & 0xffffffff;
 	$s = ((($s >> 16) & 0x0000ffff) | ($t & 0xffff0000));
@@ -441,8 +444,6 @@ sub expand_key ($) {
 }
 
 
-
-
 sub crypt ($$$) {
     my ($input, $ks, $encrypt) = @_;
     my $output;
@@ -452,17 +453,27 @@ sub crypt ($$$) {
     my ($l, $r) = unpack "VV", $input;
     usage ("at least 8 byte key") unless defined $r;
     
-    ((  $t ) = (((((( ( $r ) ) >> (  (   4 ) )) &  RIGHT_SHIFT_MASK->[(  (   4 ) )]) ) ^ (  $l )) & (  0x0f0f0f0f )), (  $l ) ^= (  $t ), ( $r ) ^= ((  $t ) << (   4 ))) ;
-    ((  $t ) = (((((( ( $l ) ) >> (  (  16 ) )) &  RIGHT_SHIFT_MASK->[(  (  16 ) )]) ) ^ (  $r )) & (  0x0000ffff )), (  $r ) ^= (  $t ), ( $l ) ^= ((  $t ) << (  16 ))) ;
-    ((  $t ) = (((((( ( $r ) ) >> (  (   2 ) )) &  RIGHT_SHIFT_MASK->[(  (   2 ) )]) ) ^ (  $l )) & (  0x33333333 )), (  $l ) ^= (  $t ), ( $r ) ^= ((  $t ) << (   2 ))) ;
-    ((  $t ) = (((((( ( $l ) ) >> (  (   8 ) )) &  RIGHT_SHIFT_MASK->[(  (   8 ) )]) ) ^ (  $r )) & (  0x00ff00ff )), (  $r ) ^= (  $t ), ( $l ) ^= ((  $t ) << (   8 ))) ;
-    ((  $t ) = (((((( ( $r ) ) >> (  (   1 ) )) &  RIGHT_SHIFT_MASK->[(  (   1 ) )]) ) ^ (  $l )) & (  0x55555555 )), (  $l ) ^= (  $t ), ( $r ) ^= ((  $t ) << (   1 ))) ;
+    $t = (($r >> 4) ^ $l) & 0x0f0f0f0f;
+    $l ^= $t;
+    $r ^= $t << 4;
+    $t = (($l >> 16) ^ $r) & 0x0000ffff;
+    $r ^= $t;
+    $l ^= $t << 16;
+    $t = (($r >> 2) ^ $l) & 0x33333333;
+    $l ^= $t;
+    $r ^= $t << 2;
+    $t = (($l >> 8) ^ $r) & 0x00ff00ff;
+    $r ^= $t;
+    $l ^= $t << 8;
+    $t = (($r >> 1) ^ $l) & 0x55555555;
+    $l ^= $t;
+    $r ^= $t << 1;
     
     $t = ($r << 1) | (($r >> 31) & 0x1);
     $r = ($l << 1) | (($l >> 31) & 0x1);
     $l = $t;
     
-    #Clear the top bits on machines with 8byte longs.
+    # Clear the top bits on machines with 8byte longs.
     $l &= 0xffffffff;
     $r &= 0xffffffff;
     
@@ -471,32 +482,82 @@ sub crypt ($$$) {
     
     if ($encrypt) {
 	for ($i = 0; $i < 32; $i += 4) {
-	      $u   = (  $r  ^   $s [  $i + 0   ]);   $t   =    $r  ^   $s [  $i + 0 +1];   $t   = ((((   $t  ) >> (  4 )) &  RIGHT_SHIFT_MASK->[(  4 )])  + (  $t  << 28));  $l  ^= des_SPtrans_x ->[1][(  $t       ) & 0x3f]| des_SPtrans_x ->[3][(  $t  >>  8) & 0x3f]| des_SPtrans_x ->[5][(  $t  >> 16) & 0x3f]| des_SPtrans_x ->[7][(  $t  >> 24) & 0x3f]| des_SPtrans_x ->[0][(  $u       ) & 0x3f]| des_SPtrans_x ->[2][(  $u  >>  8) & 0x3f]| des_SPtrans_x ->[4][(  $u  >> 16) & 0x3f]| des_SPtrans_x ->[6][(  $u  >> 24) & 0x3f]; ;
-	      $u   = (  $l  ^   $s [  $i + 2   ]);   $t   =    $l  ^   $s [  $i + 2 +1];   $t   = ((((   $t  ) >> (  4 )) &  RIGHT_SHIFT_MASK->[(  4 )])  + (  $t  << 28));  $r  ^= des_SPtrans_x ->[1][(  $t       ) & 0x3f]| des_SPtrans_x ->[3][(  $t  >>  8) & 0x3f]| des_SPtrans_x ->[5][(  $t  >> 16) & 0x3f]| des_SPtrans_x ->[7][(  $t  >> 24) & 0x3f]| des_SPtrans_x ->[0][(  $u       ) & 0x3f]| des_SPtrans_x ->[2][(  $u  >>  8) & 0x3f]| des_SPtrans_x ->[4][(  $u  >> 16) & 0x3f]| des_SPtrans_x ->[6][(  $u  >> 24) & 0x3f]; ;
+	    $u  = ($r ^ $s[($i + 0)    ]);
+	    $t =   $r ^ $s[($i + 0) + 1];
+	    $t = (($t >> 4) & 0x0fffffff) | ($t << 28);
+	    $l ^= des_SPtrans_1->[($t      ) & 0x3f]|
+	          des_SPtrans_3->[($t >>  8) & 0x3f]|
+	          des_SPtrans_5->[($t >> 16) & 0x3f]|
+	          des_SPtrans_7->[($t >> 24) & 0x3f]|
+	          des_SPtrans_0->[($u      ) & 0x3f]|
+	          des_SPtrans_2->[($u >>  8) & 0x3f]|
+	          des_SPtrans_4->[($u >> 16) & 0x3f]|
+	          des_SPtrans_6->[($u >> 24) & 0x3f];
+	    $u  = ($l ^ $s[($i + 2)    ]);
+	    $t =   $l ^ $s[($i + 2) + 1];
+	    $t = (($t >> 4) & 0x0fffffff) | ($t << 28);
+	    $r ^= des_SPtrans_1->[($t      ) & 0x3f]|
+	          des_SPtrans_3->[($t >>  8) & 0x3f]|
+	          des_SPtrans_5->[($t >> 16) & 0x3f]|
+	          des_SPtrans_7->[($t >> 24) & 0x3f]|
+	          des_SPtrans_0->[($u      ) & 0x3f]|
+	          des_SPtrans_2->[($u >>  8) & 0x3f]|
+	          des_SPtrans_4->[($u >> 16) & 0x3f]|
+	          des_SPtrans_6->[($u >> 24) & 0x3f];
 	}
     } else {
 	for ($i = 30; $i > 0; $i -= 4) {
-	      $u   = (  $r  ^   $s [  $i - 0   ]);   $t   =    $r  ^   $s [  $i - 0 +1];   $t   = ((((   $t  ) >> (  4 )) &  RIGHT_SHIFT_MASK->[(  4 )])  + (  $t  << 28));  $l  ^= des_SPtrans_x ->[1][(  $t       ) & 0x3f]| des_SPtrans_x ->[3][(  $t  >>  8) & 0x3f]| des_SPtrans_x ->[5][(  $t  >> 16) & 0x3f]| des_SPtrans_x ->[7][(  $t  >> 24) & 0x3f]| des_SPtrans_x ->[0][(  $u       ) & 0x3f]| des_SPtrans_x ->[2][(  $u  >>  8) & 0x3f]| des_SPtrans_x ->[4][(  $u  >> 16) & 0x3f]| des_SPtrans_x ->[6][(  $u  >> 24) & 0x3f]; ;
-	      $u   = (  $l  ^   $s [  $i - 2   ]);   $t   =    $l  ^   $s [  $i - 2 +1];   $t   = ((((   $t  ) >> (  4 )) &  RIGHT_SHIFT_MASK->[(  4 )])  + (  $t  << 28));  $r  ^= des_SPtrans_x ->[1][(  $t       ) & 0x3f]| des_SPtrans_x ->[3][(  $t  >>  8) & 0x3f]| des_SPtrans_x ->[5][(  $t  >> 16) & 0x3f]| des_SPtrans_x ->[7][(  $t  >> 24) & 0x3f]| des_SPtrans_x ->[0][(  $u       ) & 0x3f]| des_SPtrans_x ->[2][(  $u  >>  8) & 0x3f]| des_SPtrans_x ->[4][(  $u  >> 16) & 0x3f]| des_SPtrans_x ->[6][(  $u  >> 24) & 0x3f]; ;
+	    $u  = ($r ^ $s[($i - 0)    ]);
+	    $t =   $r ^ $s[($i - 0) + 1];
+	    $t = (($t >> 4) & 0x0fffffff) | ($t << 28);
+	    $l ^= des_SPtrans_1->[($t      ) & 0x3f]|
+	          des_SPtrans_3->[($t >>  8) & 0x3f]|
+	          des_SPtrans_5->[($t >> 16) & 0x3f]|
+	          des_SPtrans_7->[($t >> 24) & 0x3f]|
+	          des_SPtrans_0->[($u      ) & 0x3f]|
+	          des_SPtrans_2->[($u >>  8) & 0x3f]|
+	          des_SPtrans_4->[($u >> 16) & 0x3f]|
+	          des_SPtrans_6->[($u >> 24) & 0x3f];
+	    $u  = ($l ^ $s[($i - 2)    ]);
+	    $t =   $l ^ $s[($i - 2) + 1];
+	    $t = (($t >> 4) & 0x0fffffff) | ($t << 28);
+	    $r ^= des_SPtrans_1->[($t      ) & 0x3f]|
+	          des_SPtrans_3->[($t >>  8) & 0x3f]|
+	          des_SPtrans_5->[($t >> 16) & 0x3f]|
+	          des_SPtrans_7->[($t >> 24) & 0x3f]|
+	          des_SPtrans_0->[($u      ) & 0x3f]|
+	          des_SPtrans_2->[($u >>  8) & 0x3f]|
+	          des_SPtrans_4->[($u >> 16) & 0x3f]|
+	          des_SPtrans_6->[($u >> 24) & 0x3f];
 	}		
     }
     
     $l = (($l >> 1) & 0x7fffffff) | ($l << 31);
     $r = (($r >> 1) & 0x7fffffff) | ($r << 31);
-    #Clear the top bits on machines with 8byte longs.
+    # Clear the top bits on machines with 8byte longs.
     $l &= 0xffffffff;
     $r &= 0xffffffff;
     
-    #Swap $l and $r.
-    #We will not do the swap so just remember they are
-    #Reversed for the rest of the subroutine
-    #Luckily FP fixes this problem :-)
+    # Swap $l and $r.
+    # We will not do the swap so just remember they are
+    # Reversed for the rest of the subroutine
+    # Luckily FP fixes this problem :-)
     
-    ((  $t ) = (((((( ( $r ) ) >> (  (   1 ) )) &  RIGHT_SHIFT_MASK->[(  (   1 ) )]) ) ^ (  $l )) & (  0x55555555 )), (  $l ) ^= (  $t ), ( $r ) ^= ((  $t ) << (   1 ))) ;
-    ((  $t ) = (((((( ( $l ) ) >> (  (   8 ) )) &  RIGHT_SHIFT_MASK->[(  (   8 ) )]) ) ^ (  $r )) & (  0x00ff00ff )), (  $r ) ^= (  $t ), ( $l ) ^= ((  $t ) << (   8 ))) ;
-    ((  $t ) = (((((( ( $r ) ) >> (  (   2 ) )) &  RIGHT_SHIFT_MASK->[(  (   2 ) )]) ) ^ (  $l )) & (  0x33333333 )), (  $l ) ^= (  $t ), ( $r ) ^= ((  $t ) << (   2 ))) ;
-    ((  $t ) = (((((( ( $l ) ) >> (  (  16 ) )) &  RIGHT_SHIFT_MASK->[(  (  16 ) )]) ) ^ (  $r )) & (  0x0000ffff )), (  $r ) ^= (  $t ), ( $l ) ^= ((  $t ) << (  16 ))) ;
-    ((  $t ) = (((((( ( $r ) ) >> (  (   4 ) )) &  RIGHT_SHIFT_MASK->[(  (   4 ) )]) ) ^ (  $l )) & (  0x0f0f0f0f )), (  $l ) ^= (  $t ), ( $r ) ^= ((  $t ) << (   4 ))) ;
+    $t = (($r >> 1) ^ $l) & 0x55555555;
+    $l ^= $t;
+    $r ^= $t << 1;
+    $t = (($l >> 8) ^ $r) & 0x00ff00ff;
+    $r ^= $t;
+    $l ^= $t << 8;
+    $t = (($r >> 2) ^ $l) & 0x33333333;
+    $l ^= $t;
+    $r ^= $t << 2;
+    $t = (($l >> 16) ^ $r) & 0x0000ffff;
+    $r ^= $t;
+    $l ^= $t << 16;
+    $t = (($r >> 4) ^ $l) & 0x0f0f0f0f;
+    $l ^= $t;
+    $r ^= $t << 4;
     
     pack "VV", $l, $r;
 }
@@ -526,15 +587,7 @@ Algorithm  (DEA) is a semi-strong encryption and decryption algorithm.
 
 The module is 100 % compatible to Crypt::DES but is implemented 
 entirely in Perl.  That means that you do not need a C compiler to 
-build and install this extension.  However, you will need a C 
-preprocessor, notably cpp(1), to hack on the module sources  because 
-for performance reasons the module is run through the preprocessor 
-to inline some function calls.  There is no effort made to find out 
-how the preprocessor is run on your system.  If you want to hack on
-the module, make sure that a cpp that writes to stdout is the first 
-cpp that is found in your PATH.  is not really Perl, you need to run it through the preprocessor.  is of no concern if you just 
-want to use the extension because the preprocessor output is included 
-in the distribution.
+build and install this extension.  
 
 The module implements the Crypt::CBC interface.  You are encouraged
 to read the documentation for Crypt::CBC if you intend to use this
@@ -545,25 +598,23 @@ cause an exception, longer keys will get silently truncated.  Data
 is encrypted and decrypted in blocks of 8 bytes.
 
 The module implements the Ultra-Fast-Crypt (UFC) algorithm as found
-for example in the GNU libc.  
+for example in the GNU libc.  On the Perl side a lot has been done
+in order to make the module as fast as possible (function inlining,
+use integer, ...).
+
+Note: For performance issues the source code for the module is
+first preprocessed by m4.  That means that you need an m4 macro
+processor in order to hack on the sources.  This is of no concern
+for you if you only want to use the module, the preprocessed output
+is always included in the distribution.
 
 =head1 BUGS
 
-Nothing known.  The module has not been tested on 64 bit or big-endian 
-architectures.
-
-=head1 CAVEATS
-
-The XS version of DES Crypt::DES(3) outperforms this pure Perl version
-by approximately factor 35 (with cached keys) resp. factor 13 (with
-non-cached keys).  In Cipher Block Chaining mode, while encrypting
-a large plaintext, the XS version outperforms Crypt::DES_PP by
-approxymately factor 20 here, but only by factor 9 when encrypting
-128 byte blocks with varying keys.  Judge yourself.
+Nothing known.  The module has not been tested on 64 bit architectures.
 
 =head1 AUTHOR
 
-is not really Perl, you need to run it through the preprocessor.  implementation was written by Guido Flohr (guido@imperia.net).
+This implementation was written by Guido Flohr (guido@imperia.net).
 It is available under the terms of the Lesser GNU General Public
 License (LGPL) version 2 or - at your choice - any later version,
 see the file ``COPYING.LIB''.
@@ -575,7 +626,17 @@ license version 2, see the file ``COPYING.LIB''.
 
 =head1 SEE ALSO
 
-Crypt::CBC(3), Crypt::DES(3), perl(1).
+Crypt::CBC(3), Crypt::DES(3), perl(1), m4(1).
 
 =cut
 
+Local Variables:
+mode: perl
+perl-indent-level: 4
+perl-continued-statement-offset: 4
+perl-continued-brace-offset: 0
+perl-brace-offset: -4
+perl-brace-imaginary-offset: 0
+perl-label-offset: -4
+tab-width: 4
+End:                                                                            
